#include "C:\Programme\Atmel\AVR Tools\AvrAssembler2\Appnotes\tn2313def.inc"
;****************************************************************************************
;*	Uhr, Kalender & Thermometer															*
;*																						*
;*	Hardware: AT-Tiny2313 mit ext. 8MHz Quarz											*
;*	HD44780 LCD 16*2 mit LED Hintergrundbeleuchtung										*
;*	AVR	-	LCD																			*
;*	--------------																		*
;*	Pd0	-	DB4																			*
;*	Pd1	-	DB5																			*
;*	Pd2	-	DB6																			*
;*	Pd3	-	DB7																			*
;*	Pd4	-	RS																			*
;*	Pd5	-	E																			*
;*	GND	-	R/W	(*nur reinschreiben und die max. Zeit abwarten)							*
;*	NC 	-	DB0-DB3																		*
;*	LED-Backlight ber NPN Transistor an PB5											*
;*	Drehgeber mit Taster. Verbunden mit PB2 (Taster) und PB3/PB4 Drehgeber				*
;*	TSic206 Themperatursensor. PinB0 Daten, PB1 VCC (Sensor ein/ausschalten)			*															*
;*	Summer an PB7																		*
;*																						*
;*	Autor: Jrgen Woetzel www.avr-projekte.de											*
;*	Version vom 17.06.11																*
;****************************************************************************************
.def	null	=	r0
.def	sechzig	=	r11
.def	vierundzwanzig	=	r12
.def	second	=	r14
.def	stat	=	r15
.def	richtung =	r13
.def	taste	=	r2

.def	sek		=	r4
.def	min		=	r5
.def	stu		=	r6
.def	tag		=	r7
.def	mon		=	r8
.def	jahr	=	r9
.def	wtag	=	r10

.def	amin	=	r3
.def	astu	=	r17

.def	temp1	=	r18
.def	temp2	=	r19
.def	temp3	=	r20
.def	temp4	=	r21
.def	IRQ_cnt	=	r22
.def	Enc		=	r23
.def	rawlow	=	r30
.def	rawhigh =	r31

.def	pwmcnt	=	r16
.def	pwm		=	r1

.def	flags		=	r24
.equ	error		=	7
.equ	key			=	6
.equ	schaltjahr	=	5
.equ	winterzeit	=	4
.equ	minus		=	3
.equ	wecken		=	2
.equ	buzz		=	1
.equ	bit8		=	0
;--------------------------------------------------------------------------
;Makros
;--------------------------------------------------------------------------
.macro	print
		ldi		zl,low(@0*2)				;Text auf LCD
		ldi		zh,high(@0*2)
		rcall	tx1
.endmacro

.macro	cur									;Cursor auf 16*2 LCD setzen
		ldi		temp1,1<<7|@0+$40*@1		;x=0..15(Spalte), Y=0..1(Zeile)
		rcall	BefOut								
.endmacro

.macro	ldx
		ldi		xl,low(@0)
		ldi		xh,high(@0)
.endmacro
;--------------------------------------------------------------------------
;SRamadressen
;--------------------------------------------------------------------------
.dseg
tstring:	.byte	8		;Temperaturstring		
zstring:	.byte	10		;Uhr
dstring:	.byte	18		;Datum
;--------------------------------------------------------------------------
.cseg
.org	0
		rjmp	init
.org	OC1Aaddr			;Timer0 Overflow Interrupt
;		rjmp	Timer_int
;**************************************************************************
;							Timer Interrupt
;**************************************************************************
Timer_Int:
		in		stat,SREG			;Status sichern
		push	temp1
	;---------------------------------------------------------------------
	;ggf. Summer schalten
	;---------------------------------------------------------------------
		sbrs	flags,buzz			;Summer ein?
		rjmp	isr00				;Bei nein, Sprung zum weiteren Verlauf der ISR
		cp		sek,vierundzwanzig	;Nach 24 Sekunden Alarm, doppelte Frequenz (= mehr Lrm)
		brcc	stress
		sbrs	flags,bit8			;Bit toggelt alle 500 mS
		rjmp	isr00				;Dadurch bekommt der Summer eine Rechteckspannung von 1 Hz 
stress:	mov		temp1,sek			;Die Tne mit zunehmender Zeit, verlngern
		lsl		temp1
		lsl		temp1				;Sekunden*4 ergibt max. 236. IRQ_CNT = max. 250
		cp		irq_cnt,temp1		;Das Puls- Pausenverh. = 4:250 (bei 1 sek.) - 236:250 (bei 59 sek.)
		brcc	isr00
		cbi		portb,7				;Summer ein
		rjmp	isr01
isr00:	sbi		portb,7				;Summer aus
	;-----------------------------------------------------------------------
	;LED-Beleuchtung minimal-PWM mit 8 Stufen
	;luft mit (500/8) 62,5 Hz, also hart an der Grenze		
	;-----------------------------------------------------------------------
isr01:
		inc		pwmcnt				;pwmzhler
		andi	pwmcnt,7			;Zhlt von 0..7
		cp		pwmcnt,pwm			;Vergleich von Zhler und Variable
		brcs	isr02
		cbi		portb,5				;Backlight aus				
		rjmp	isr03
isr02:	sbi		portb,5				;Backlight ein

	;-----------------------------------------------------------------------
	;500 ISRs zhlen, dann sekunde erhhen
	;-----------------------------------------------------------------------

isr03:	inc		IRQ_cnt				;Zaehlt die IRQs
		cpi		IRQ_cnt,250			; /500 = 1Hz
		brne	isr05
		clr		IRQ_cnt				;wenn 250, wieder bei 0 anfangen
		sbrs	flags,bit8			;2* bis 250 zhlen = genau 1 Sekunde			
		rjmp	isr04
		cbr		flags,1<<bit8
		rjmp	isr05
isr04:	sbr		flags,1<<bit8
		inc		second				;Sekunde +1, wird im Hauptprogramm nach Verarbeitung wieder gelscht 

	;---------------------------------------------------------------------------
	;Drehgeber einlesen, ist verbunden mit PinB2 (Taster) und PinB3/PinB4 Drehgeber
	;---------------------------------------------------------------------------
isr05:	in		temp1,pinb		;pinb holen
		ror		temp1			;PB0 und PB1 rechts rausschubsen	
		ror		temp1					
		;----------------------------------------------------------------------
		;Taste am Drehgeber entprellen
		;----------------------------------------------------------------------
		ror		temp1			;Taste ist im Carry
		rol		taste			;Zustand der letzten 8 ISRs von rechts nach links
		brcc	isr06			;Wenn vor 9 ISRs gedrckt war, Sprung
		brne	isr06			;... wenn nicht die letzten 8 ISR gedrckt war, Sprung
		sbr		flags,1<<key	;Wenn die letzten 9 ISR 100000000 (C+taste) Flag setzen
		;----------------------------------------------------------------------
		;Drehgeber einlesen
		;----------------------------------------------------------------------
isr06:	ror		temp1			;pinb3 ins Carry...
		rol		enc				;.. und in bit0 
		ror		temp1			;pinb4 ins Carry
		rol		enc				;und nochmal schieben
		andi	enc,$0f			;das Bitmuster der letzten ISR steht nun in bit2 und 3
;		cpi		enc,0b1011		;bei den folgenden 4 kombis wurde nach links gedreht.
;		breq	links
;		cpi		enc,0b1101		;Nur jeden 4. Flankenwechsel auswerten. Wenn der Drehgeber zu langsam ist
;		breq	links			;Auskommentierungen entfernen
		cpi		enc,0b0100
		breq	links
;		cpi		enc,0b0010
;		breq	links
;		cpi		enc,0b0001		;rechts
;		breq	rechts
; 		cpi		enc,0b0111
; 		breq	rechts
;		cpi		enc,0b1110
;		breq	rechts
		cpi		enc,0b1000
		breq	rechts
		rjmp	endti			;bei allen anderen Kominationen wurde nicht gedreht,								;oder Fehler (zu schnell)
links:	inc		richtung
		rjmp	endti
rechts:	dec		richtung
	;---------------------------------------------------------------------------
	;ENDE Drehgeber
	;---------------------------------------------------------------------------
endti:
		pop		temp1
		out		SREG,stat		;Status wieder zurck
		reti		
;**************************************************************************
;						 Timer Interrupt ENDE
;**************************************************************************
init:
	ldi 	temp1, RAMEND				;Stack
	out 	SPL, temp1
	ldi		temp1,1<<pb1|1<<pb5|1<<pb7	;Tsic, VCC/Backlight, PWM/Buzzer als Ausgang
	out		ddrb,temp1
	ldi		temp1,0b00011100			;Pullups Encoder ein / Sensor aus
	out		portb,temp1
	ldi		temp1,$8					;Backlight Startwert (100%)
	mov		pwm,temp1
	;Konstante 
	ldi		temp1,60
	mov		sechzig,temp1
	ldi		temp1,24
	mov		vierundzwanzig,temp1
	clr		null
	;Variable
	clr		richtung
	clr		second
	clr		flags
	;Startwerte Uhr & Kalender
	mov		sek,null
	mov		min,null
	mov		stu,null
	mov		tag,null
	inc		tag
	mov		mon,null
	mov		jahr,null
	rcall	wochentag
	;Startwert Alarm
	mov		astu,null
	mov		amin,null
	;---------------------------------------------------
	;LCD - init
	;---------------------------------------------------
	rcall	lcd_init				;Init nach Power Up		
	ldi		temp1,8|1<<6			;ins CG-Ram ab Sonderzeichen 1
	rcall	befout
	ldi		xl,24					;8 Bytes * 3 Zeichen
	ldi		zl,low(Ch1<<1)			;Sonderzeichentab im Flash
	ldi		zh,high(Ch1<<1)
mch:lpm		temp1,z+				;Bitmaps der 3 Chars zum LCD schicken
	rcall	ascout
	dec		xl
	brne	mch
	ldi		temp1,1<<7				;wieder	auf Textmodus umschalten
	rcall	BefOut
	rcall	clrs					;LCD lschen, Cursor oben links
	;---------------------------------------------------
	;Timerinterupt anwerfen
	;---------------------------------------------------
	ldi		temp1,high(16000-1)		;Vergleichswert =16000 ergibt
	out		OCR1AH,temp1			;bei 8MHz genau 500 Hz
	ldi		temp1,low(16000-1)		;Bei Korrektur der Genauigkeit ergibt +1 ca -2,3 Sek. pro Tag
	out		OCR1AL,temp1
	ldi		temp1,(1<<WGM12)|(1<<CS10);Prescaler=1 Timer Compare ein
	out		tccr1b,temp1
	ldi		temp1,1<<ocie1a			;Interrupt bei Timer Compare ein
	out		TIMSK,temp1				
	sei
;**************************************************************************
;							 Mainloop
;**************************************************************************
main:
	sbrc	flags,key				;Taste am Drehgeber gedrckt ?
	rjmp	Button					;Wenn ja, Sprung
	rcall	dimm					;Nachschauen ob gedreht wurde & evtl. Backlight dimmen
	tst		second					;Ist Zeit vergangen ?
	breq	main					
	rcall	inctime					;1 Sekunde hochzhlen (mit allen Konsequenzen:-))
	rcall	gettemp					;Temperatur abfragen und aufs LCD
	rjmp	main
;**************************************************************************
;							 Mainloop ENDE
;**************************************************************************
;Minimaldimmer. Backlight in 8 Stufen dimmen.
;--------------------------------------------------------------------------
dimm:
	tst		richtung	;Dreht da einer ?
	breq	bamain		;Bei nein, Sprung
	mov		temp1,pwm
	brpl	heller		;Nach rechts gedreht, heller

	cpi		temp1,0		;Gehts noch dunkler ?
	breq	bamain
	dec		pwm			;Eine Stufe dunkler
	rjmp	bamain
heller:
	cpi		temp1,8
	breq	bamain
	inc		pwm			;Eine Stufe heller
bamain:
	clr		richtung
	ret
;****************************************************************************
;						Uhr um eine Sekunde hochzhlen
;****************************************************************************
inctime:
	dec		second					;Von der ISR hochgezhlte Sekunde abziehen				
	inc		sek						;Sekunde +1
	cp		sek,sechzig				;Minute voll ?
	brne	retmain					;Wenn nein, Sprung
	clr		sek						;Sekunde=0
	inc		min						;Minute+1
	rcall	testalarm				;Wenn Alarm aktiviert ist, ggf. Summer schalten
	cp		min,sechzig				;Wie gehabt
	brne	retmain
	clr		min
	inc		stu						;Stunde+1
	rcall	testalarm				;Bei voller Stunde Alarm nochmal testen
	ldi		temp1,2
	cp		stu,temp1				;cpi Stunden,2
	breq	testsommerzeit			;02:00:00 Test Sommerzeit
	inc		temp1
	cp		stu,temp1				;cpi Stunden,3
	breq	testwinterzeit			;03:00:00 Test Winterzeit
	cp		stu,vierundzwanzig
	brne	retmain
	cbr		flags,1<<winterzeit
	clr		stu						;00:00:00 Kalender +1
	rcall	testalarm				;Hat da jemand den Alarm genau auf 00:00:00?
	;****************************************************************************
	;							Kalender
	;****************************************************************************
it1:ldi		zl,low(molen*2)			;Zeiger auf Tab. (Tage pro Monat)
	ldi		zh,high(molen*2)
	add		zl,mon					;+ aktueller Monat
	adc		zh,null
	lpm		temp2,z					;Anzahl der Tage im Monat -> temp2
	inc		tag
	cp		temp2,tag				;Sind wir schon drber ?
	brpl	retmain					;Wenn nein, Sprung
	clr		tag
	inc		tag						;Es ist der 1.
	inc		mon						;Monat +1
	ldi		temp1,12
	cp		temp1,mon				;cpi	monat,12
	brne	retmain
	clr		mon
	inc		jahr					;Jahr +1
	mov		temp1,jahr
	cpi		temp1,100				;Glaube nicht, das das vorkommt ;-)
	brne	retmain
	clr		jahr
retm1:
	cbr		flags,1<<winterzeit
retmain:
	rcall	wochentag				;Anhand des Datums den Wochentag berechnen
	rcall	timestring				;Hex -> String
	ldi		xl,low(zstring)
	ldi		xh,high(zstring)
	rcall	home
	rcall	tx2						;String -> LCD
	rcall	datestring				;dito
	ldi		xl,low(dstring)
	ldi		xh,high(dstring)
	cur		0,1
	rcall	tx2
	ret
	;----------------------------------------------------------------------------------------------
	;Die mitteleuropische Sommerzeit beginnt am letzten Sonntag im Mrz um 2 Uhr.
	;Zum Beginn der Sommerzeit wird die Uhr um eine Stunde von 2 Uhr auf 3 Uhr vorgestellt.
	;----------------------------------------------------------------------------------------------
testsommerzeit:			;Es ist 02:00:00 Uhr
	ldi		temp1,2		;2=Mrz (0=JAN)
	cp		mon,temp1
	brne	retmain
	rcall	testlastsun
	brtc	retmain
	inc		stu			;Eine Stunde Vorstellen
	rjmp	retmain
	;----------------------------------------------------------------------------------------------
	;Die mitteleuropische Sommerzeit endet jeweils am letzten Sonntag im Oktober um 3 Uhr mitteleuropischer Sommerzeit.
	;Die Uhr wird von 3 Uhr auf 2 Uhr zurckgestellt.
	;----------------------------------------------------------------------------------------------
testwinterzeit:					;Es ist 03:00:00 Uhr
	ldi		temp1,9				;9=Oktober (0=JAN)
	cp		mon,temp1
	brne	retmain				;Wenn nicht Oktober ist, Rcksprung
	sbrc	flags,winterzeit	;Haben wir vor einer Std. schon zurckgestellt ?
	rjmp	retm1				;Wenn ja
	rcall	testlastsun			;Test, letzter So im Monat
	brtc	retmain
	dec		stu					;Uhr eine Stunde zurckstellen
	sbr		flags,1<<winterzeit	;Flag, Uhr ist schon umgestellt
	rjmp	retmain
	;----------------------------------------------------------------------------------------------
	;Unterprg. Testet ob letzter Sonntag im Monat
	;T-Flag = 1, wenn ja
	;----------------------------------------------------------------------------------------------
testlastsun:
	ldi		temp1,6		;6=Sonntag (0=Mo)
	cp		wtag,temp1
	brne	tls_nein	;Wenn kein Sonntag, Rcksprung
	mov		temp1,tag
	cpi		temp1,24	;Wenn kleiner, kommt noch ein Sonntag
	brlo	tls_nein
	bset	6			;OK, es ist der letzte Sontag im Monat
	ret	
tls_nein:
	bclr	6
	ret
;**********************************************************************************************
;									*** Wecker ***
;**********************************************************************************************
testalarm:
	cp		min,amin		;Minuten vergleichen
	brne	xalarm
	cp		stu,astu		;Stunden vergleichen
	brne	xalarm
	sbrc	flags,wecken	;Ist der Alarmton eingeschaltet?
	sbr		flags,1<<buzz	;Summer ein, den Rest macht die ISR
	ret	
xalarm:
	cbr		flags,1<<buzz	;Summer aus
	ret
;**********************************************************************************************
;									*** Wecker stellen ***
;**********************************************************************************************
setalarm:
	cbr		flags,1<<key	;Stunden einlesen
	rcall	clrs
	print	alarmtxt		;" ** ALARM **"
	cur		0,1				;Text in Zeile 2
	print	txt5			;"Stunden <00>"	
	mov		temp1,astu
	ldi		temp3,0
	ldi		temp4,23
	rcall	inbyte			;Byte (0..23) ber Drehencoder Einlesen
	mov		astu,temp1
	cbr		flags,1<<key	;Minuten einlesen
	cur		0,1
	print	txt6			;"Minuten <00>
	mov		temp1,amin
	ldi		temp3,0
	ldi		temp4,59
	rcall	inbyte			;Byte (0..59) ber Drehencoder Einlesen
	mov		amin,temp1
	cbr		flags,1<<key
onoff:						;Alarmton Ein/Aus
	clr		richtung		;Drehgeberwert lschen
	cur		0,1
	sbrc	flags,wecken	;Dieses Bit ber Drehencoder toggeln 
	rjmp	bon
	print	txt9			;"Alarmton <AUS>"
	rjmp	bof
bon:print	txt8			;"Alarmton <EIN>"
bof:sbrc	flags,key
	rjmp	exalarm			;Wenn Taste am Drehgeber gedrckt wurde, fertig
	tst		richtung		;Wurde gedreht ?
	breq	bof				;Wenn nein, neue Runde
	ldi		temp1,1<<wecken
	eor		flags,temp1		;Alarmbit toggeln
	rjmp	onoff
exalarm:
	cbr		flags,1<<key
	rcall	clrs
	rjmp	main
;**********************************************************************************************
;                               ***  ENDE WECKER ***
;**********************************************************************************************
;---------------------------------------------------------------------------
;Taste am Drehgeber wurde gedrckt
;Es gibt 3 Mglichkeiten. Kurz, lang, 2*kurz Hintereinander (Doppelklick)	
;---------------------------------------------------------------------------
Button:
	sbrc	flags,buzz			;Wenn der Summer lrmt, diesen abschalten
	rjmp	buzzoff
	cbr		flags,1<<key		;Tastenflag lschen
	rcall	wait1200			;1,2 Sekunden warten
	sbrc	flags,key			;Bei Doppelclick auf Taster, Wecker stellen
	rjmp	setalarm
	tst		taste				;Bei langem Tastendruck, Uhrzeit/Datum stellen
	breq	settime
buzzoff:
	cbr		flags,1<<key|1<<buzz	;Tastenflag und Summer lschen
	rjmp	main
;**********************************************************************************************
;                                   *** SETUP ***
;								Uhr und Datum stellen
;**********************************************************************************************
	;---------------------------------------------------
	;Jahr einstellen
	;---------------------------------------------------
settime:
	rcall	clrs				;Display lschen
	print	Menutxt				;" *** Setup *** "
	cur		0,1					;Cursor 2.Zeile
	print	txt1				;"Jahr <20  >" das hlt bis 2099 :-)
	mov		temp1,jahr			;In hohes Register kopieren
	ldi		temp3,0				;Min./ Max. Wert fr Inbyte UPrg.
	ldi		temp4,99
	rcall	inbyte				;Liest einen Wert ber Drehencoder ein
	mov		jahr,temp1			;Zurcklesen
	rcall	test_sj				;Auf Schaltjahr testen und ggf. Flag setzen
	;---------------------------------------------------
	;Monat
	;---------------------------------------------------
	cbr		flags,1<<key		;dito
	rcall	clrs
	print	Menutxt
	cur		0,1
	print	txt2
	mov		temp1,mon
	inc		temp1
	ldi		temp3,1
	ldi		temp4,12
	rcall	inbyte
	dec		temp1
	mov		mon,temp1
	;---------------------------------------------------
	;Tag
	;---------------------------------------------------
	cbr		flags,1<<key
	rcall	clrs
	print	Menutxt
	cur		0,1
	print	txt3	
	ldi		zl,low(molen<<1)		;Wieviele Tage hat der Monat
	ldi		zh,high(molen<<1)		;Zeiger auf Tabelle	im Flash
	add		zl,mon					;Zeiger auf aktuellen Monat
	adc		zh,null
	lpm		temp4,z					;Max. setzen
	sbrs	flags,schaltjahr		;Haben wir ein Schaltjahr ?
	rjmp	tag1					;Wenn nein, Sprung
	mov		temp1,mon
	cpi		temp1,1					;Wenn Schaltjahr, ist Februar ?
	brne	tag1
	inc		temp4					;Wenn ja max+1 (29 Tage)
Tag1:
	mov		temp1,tag
	cp		temp1,temp4				;Alter Wert > max.dieser Monat ?			
	brcs	tag2					;Bei nein Sprung
	mov		temp1,temp4				;temp4 = max fr Inbyte UPrg.
Tag2:
	ldi		temp3,1					;Min.
	rcall	inbyte
	mov		tag,temp1
	cbr		flags,1<<key
	rcall	wochentag				;Aus Datum Wochentag berechnen
	;---------------------------------------------------
	;Stunden
	;---------------------------------------------------
	cbr		flags,1<<key
	rcall	clrs
	print	Menutxt
	cur		0,1
	print	txt5
	mov		temp1,stu
	ldi		temp3,0
	ldi		temp4,23
	rcall	inbyte
	mov		stu,temp1
	cbr		flags,1<<key
	;---------------------------------------------------
	;Minuten
	;Hier wird hh:mm:ss angezeigt, und nach Tastendruck
	;die Uhr mit den eingestellten Werten gestartet
	;---------------------------------------------------
	rcall	clrs
	print	Menutxt		;"** Setup **"
	cur		0,1			;2. Zeile
	print	txt7		;"<00:00:00> <Los>"
	clr		sek			;Sekunden lschen beim Uhr stellen
	clr		richtung	;Drehgeber lschen
nxt:
	rcall	timestring			;Zeitvariable in String wandeln	
	ldi		xl,low(zstring)
	ldi		xh,high(zstring)
	cur		1,1					;Cursor nach dem < plazieren
	rcall	tx2					;Zeit ausgeben
nxtt:
	tst		richtung
	brne	uptime				;Wenn gedreht wurde, Sprung

	sbrc	flags,key			;Wenn gedrckt wurde, Uhr starten
	rjmp	bmain
	rjmp	nxtt
	;Zurck zum Mainprg. und Uhr starten
bmain:
	cli
	clr		irq_cnt					;Sekunde lschen
	cbr		flags,1<<key|1<<bit8
	clr		second
	sei
	rcall	clrs					;LCD lschen
	rjmp	main					;und tschss
	;--------------------------------------------------------------------------
	;Zeit "hh:mm:ss" ber Drehgeber einstellen
	;--------------------------------------------------------------------------
uptime:
	tst		richtung			;In welche Richtung wurde gedreht?		
	brmi	dwn
	dec		richtung									
	inc		min					;Minuten +1
	cp		min,sechzig
	brne	nxt
	clr		min					;Bei 60, Minuten=0 Stunden+1
	inc		stu
	cp		stu,vierundzwanzig
	brne	nxt
	clr		stu					;Bei 24, Stunden =0
	rjmp	nxt
dwn:
	inc		richtung			;Wie oben, nur andere Richtung
	dec		min
	tst		min
	brpl	nxt
	mov		min,sechzig
	dec		min
	dec		stu
	tst		stu
	brpl	nxt
	mov		stu,vierundzwanzig
	dec		stu
	brne	nxt
	clr		stu
	rjmp	nxt
;   ----  ENDE SETUP ----
;----------------------------------------------------------------------------
;Wochentag berechnen
;Quelle http://userpage.fu-berlin.de/~kweinert/mathematik/wochentag.pdf
;Wochencode = Jahrhundertcode + Jahrescode + Monatscode + Tag im Monat mod 7
;----------------------------------------------------------------------------
Wochentag:
	rcall	test_sj				;Wenn Schaltjahr, Flag setzen
	ldi		temp4,6				;Jahrhundert + tabindex
	mov		temp1,jahr			;Jahr
	lsr		temp1				;mod4
	lsr		temp1
	add		temp4,temp1
	add		temp4,jahr			;+Jahr
	ldi		zl,low(monziffer<<1);Zeiger Tabelle Monatsziffer
	ldi		zh,high(monziffer<<1)
	add		zl,mon				;Monat dazu
	adc		zh,null
	lpm		temp1,z				;Monatsziffer von Tab lesen
	add		temp4,temp1			;und addieren
	add		temp4,tag			;Tag addieren
	sbrs	flags,schaltjahr	;Im Schaltjahr bei JAN & FEB -1 Tag	
	rjmp	rmod
	mov		temp1,mon
	cpi		temp1,2				;Januar oder Februar setzen das Carry
	sbci	temp4,0
rmod:							;Modulo7
	subi	temp4,7				;-7
	brcc	rmod				;ist noch was da ?
	subi	temp4,-7			;+7
	mov		wtag,temp4			;Wochentag in Variable
	ret
;----------------------------------------------------------------------------
;Auf Schaltjahr testen und Flag setzen
;----------------------------------------------------------------------------
test_sj:
	mov		temp1,jahr			;Ab 2000 (00) alle 4 Jahre
	andi	temp1,0b00000011	;Eine Zahl ist durch 4 teilbar wenn ...
	breq	sj1					;... Bit0 und Bit1 = 0
	cbr		flags,1<<schaltjahr
	ret
sj1:sbr		flags,1<<schaltjahr
	ret
;----------------------------------------------------------------------------
;String fr Datum -> SRam
;----------------------------------------------------------------------------
datestring:
	ldi		xl,low(dstring)	;Da soll der String hin
	ldi		xh,high(dstring)
	;----------------------------------------------------------------------------
	;Wochentag
	;----------------------------------------------------------------------------
	ldi		zl,low(wttext*2);Z -> Wochentagtabelle im Flash
	ldi		zh,high(wttext*2)
	mov		temp1,wtag		;wtag (0..6)
	lsl		temp1			;*2
	add		zl,temp1		;Zum Zeiger addieren
	adc		zh,null
	lpm		temp1,z+		;2 ASCii Zeichen aus der Tab. zum String hinzufgen
	st		x+,temp1
	lpm		temp1,z+
	st		x+,temp1
	ldi		temp3,'.'		;Punkt
	st		x+,temp3
	;----------------------------------------------------------------------------
	;Tag
	;----------------------------------------------------------------------------
	mov		temp1,tag		;Tag (1..31)	
	rcall	splitbyte		;in 2 Chars wandeln
	st		x+,temp2		;und wegspeichern
	st		x+,temp1
	st		x+,temp3		;'.' ist noch in temp3
	;----------------------------------------------------------------------------
	;Monat
	;----------------------------------------------------------------------------
	ldi		zl,low(motext*2)	;Wie Wochentag
	ldi		zh,high(motext*2)
	mov		temp1,mon
	lsl		temp1
	add		temp1,mon			;*3
	add		zl,temp1
	adc		zh,null
	lpm		temp1,z+			;3 Zeichen (z.B. 'JAN') zum String 
	st		x+,temp1
	lpm		temp1,z+
	st		x+,temp1
	lpm		temp1,z+
	st		x+,temp1
	ldi		temp1,' '			;Leerzeichen
	st		x+,temp1
	;----------------------------------------------------------------------------
	;Jahr
	;----------------------------------------------------------------------------
	ldi		temp4,'2'			;'20' zum String
	st		x+,temp4
	ldi		temp3,'0'
	st		x+,temp3
	mov		temp1,jahr
	rcall	splitbyte
	st		x+,temp2			;Den Rest vom Jahr hinzu
	st		x+,temp1
	;----------------------------------------------------------------------------
	;Sonderzeichen 1 oder 2 fr Alarm
	;----------------------------------------------------------------------------
	ldi		temp1,' '
	st		x+,temp1
	ldi		temp1,1			;Sonderzeichen Summer Ein
	sbrs	flags,wecken
	inc		temp1			;Sonderzeichen Summer Aus
	st		x+,temp1
	ldi		temp1,0			;String terminieren
	st		x+,temp1
	ret
;---------------------------------------------------
;den String "hh:mm:ss" im SRam ablegen
;---------------------------------------------------
timestring:
	ldi		temp3,':'		;Doppelpunkt vorladen
	ldi		xl,low(zstring)	;Da soll der String hin
	ldi		xh,high(zstring)
	mov		temp1,stu		;Stunden zuerst
	rcall	ts1
	mov		temp1,min		;Minuten
	rcall	ts1
	clr		temp3			;Null fr Stringende
	mov		temp1,sek		;Sekunden
ts1:rcall	splitbyte		;Achtung Spagetticode :-)
	st		x+,temp2
	st		x+,temp1
	st		x+,temp3
	ret
	;-----------------------------
	;Byte -> ASCii (max. 2stellig)
	;-----------------------------
splitbyte:
	ser		temp2			;zehner =-1
split:
	inc		temp2			;zehner =+1
	subi	temp1,10		;so lange 10 abziehen, bis Unterlauf	
	brpl	split
	subi	temp2,-'0'		;zehner + ASCii "0"
	subi	temp1,-'0'-10	;einer + ASC "0" + die zuviel abgezogene 10
	ret
;-------------------------------
;Byte temp1 -> Display (00-99)
;-------------------------------
printbyte:
	push	temp1
	rcall	splitbyte		;Byte nach asc
	push	temp1			;Lowbyte sichern
	mov		temp1,temp2
	rcall	ascout			;Highbyte -> LCD
	pop		temp1			;Lowbyte zurck
	rcall	ascout			;und raus
	pop		temp1
	ret
;---------------------------------------
;Byte (temp1) ber Drehencoder einlesen
;min. temp3, max. temp4
;---------------------------------------
inbyte:
	clr		richtung
	cbr		flags,1<<key
ib0:
	push	temp1
	cur		10,1
	pop		temp1
	rcall	printbyte
ib1:
	sbrc	flags,key		;Button = Wert bernehmen
	ret
	tst		richtung
	breq	ib1				;Wenn nicht gedreht, noch ne Runde
	brmi	ib2				;Wenn nach links gedreht, Sprung
	;---------------------------------------
	;nach rechts gedreht, Byte +1
	;---------------------------------------
	dec		richtung
	cp		temp1,temp4		;max.?
	breq	ib1
	inc		temp1
	rjmp	ib0
	;---------------------------------------
	;nach links gedreht, Byte -1
	;---------------------------------------
ib2:
	inc		richtung
	cp		temp1,temp3		;Min. ?
	breq	ib1
	dec		temp1
	rjmp	ib0
;**********************************************************************************
;						  --- Thermometer ---
;**********************************************************************************
;-----------------------------------------------------------------
;Temperatur auslesen und aufs LCD
;-----------------------------------------------------------------
gettemp:
	sbi		portb,1				;Saft auf den TSic
	rcall	waith				;Warten bis Datenleitung auf High
	cbr		flags,1<<error		;Fehlerflag lschen
	rcall	get_byte			;High Byte holen
	mov		rawhigh,temp3
	rcall	get_byte			;Low
	mov		rawlow,temp3
	cbi		portb,1				;TSic abschalten
	rjmp	calctemp			;Aus den Rohwerten die Temperatur berechnen

;-----------------------------------------------------------------
;Datenbyte vom TSic lesen
;-----------------------------------------------------------------
get_byte:
	rcall	startbit_time		;Wie lange ist PinB0 (Daten TSic) low ?
	rcall	sample				;Datenbyte holen
	rjmp	parity				;Paritycheck
;----------------------------
sample:
	ldi		temp3,1				;Hier kommt das Datenbyte des TSIC rein			
w_msb:
	sbic	pinb,0				;Auf neg. Flanke warten
	rjmp	w_msb
	rcall	Wait_TStrobe		;TStrobe Zeit abwarten
	sec							;Datenbit ins Carry					
	sbis	pinb,0
	clc
	rol		temp3				;Datenbit in Temp3 schieben
	rcall	waith
	brcc	w_msb				;Wenn die 1 (oben, ldi temp3,1)im Carry ist, sind wir fertig
	ret
;---------------------------
waith:
	sbic	pinb,0				;Warten bis Datenleitung auf high ist
	ret
	rjmp	waith
;-----------------------------------------------------------------
;T-Strobe Zeit messen. 8 Zyklen sind bei 8 Mhz = 1S
;-----------------------------------------------------------------
startbit_time:
	sbic	pinb,0				;Auf neg. Flanke warten
	rjmp	startbit_time
	clr		temp1				;temp1 wird alle 1 S um 1 erhht
s_time:
	inc		temp1				;1 Zyklen
	nop							;1
	nop							;1
	nop							;1
	nop							;1
	sbis	pinb,0				;1 / bei skip 2
	rjmp	s_time				;2
	ret
;-----------------------------------------------------------------
;Die oben gemessene Zeit abwarten
;-----------------------------------------------------------------
Wait_TStrobe:
	mov		temp2,temp1			;TStrobe -> Temp2
	dec		temp2
W_ST1:
	nop							;Wertvolle Zeit vergeht
	nop
	nop
	nop
	nop							;5 Zyklen
	dec		temp2				;1
	brne	W_ST1				;2 / ohne Sprung 1
	ret
;-----------------------------------------------------------------
;Parittsbit holen und auf even prfen
;-----------------------------------------------------------------
parity:
	sbic	pinb,0				;Auf neg. Flanke warten
	rjmp	parity
	rcall	Wait_TStrobe		;TStrobe Zeit abwarten
	in		temp4,pinb
	cbr		temp4,$fe			;Parittsbit brig lassen
	ldi		temp2,8				;Schleifenzhler
	push	temp3				;Datenbyte
parityl1:
	rol		temp3	
	adc		temp4,null			;Alle Einsen des Datenbits zum Paritybit addieren
	dec		temp2
	brne	parityl1
	sbrc	temp4,0				;Zahl in Temp4 muss gerade (even Parity) sein
	sbr		flags,1<<error		;sonst bertragungsfehler
	rcall	waith
	pop		temp3				;Datenbyte zurcklesen
	ret
;---------------------------------------------------------------------------------------
;	11Bit Rohwert des TSic in Temperatur umrechnen
;	Originalformel aus dem Datenblatt T= (Digital_signal/2047*(HT-LT)+LT) [C]
;	LT = -50,  HT = 150 als Standardwert fr die Temperatur-Berechnung
;
;	Umgestellt fr ASM
;	t=TSic_Wert*200 /2048-50   (die 2047 um 1 erhht um besser rechnen zu knnen)
;	t=TSic_Wert*25/256-50  (gekrzt /8)
;	Da eine Stelle hinter dem Komma angezeigt wird, rechne ich die Temperatur*10 und setze
;	vor die letzte Stelle den Dezimalpunkt (Festkomma).
;
;	Temperatur*10=(TSic_Wert*250)/256-500
;---------------------------------------------------------------------------------------

calctemp:
	;Testwerte $000=-50, $199=-10, $200=0, $2ff=25, $465=60, $6fe=125, $7ff=150	
	;ldi	rawlow,low($200)
	;ldi	rawhigh,high($200)

	clr		temp1			;temp3/2/1 enthlt durch zufgen von Temp1 (0) den TSicwert * 256
	mov		temp2,rawlow
	mov		temp3,rawhigh
	lsl		rawlow			;*2
	rol		rawhigh
	add		rawlow,temp2	;*3
	adc		rawhigh,temp3
	lsl		rawlow			;*6
	rol		rawhigh
	sub		temp1,rawlow	;*256 - *6 = *250
	sbc		temp2,rawhigh	;TSic_Wert*250 steht jetzt in temp1/temp2/temp3
	sbci	temp3,0			;Durch verwerfen des LSB (temp1) wird durch 256 geteilt	
	;	subi	temp1,-128		;Runden
	;	sbci	temp2,0			;Brauch ich das bei 0,1 Grad Auflsung? N

	;	sbci	temp3,0
	subi	temp2,low(500)	;500 (50.0) abziehen
	sbci	temp3,high(500)

	;Nun steht die Temperatur(*10) in temp2 (low) und temp3 (high)
	;-------------------------------------------------------------------------------
	;ASCii String bilden und Ausgabe auf LCD. 
	;So knnte der String aussehen "123.4",0 Das sind 7 Byte incl. der 0
	;-------------------------------------------------------------------------------
	ldi		xl,low(tstring+8)	;Zieladr. im SRam
	ldi		xh,high(tstring+8)	;+8, wir fangen hinten an. 7+1 wg. Pre-Adressierung
	st		-x,null				;String	Ende
	ldi		temp1,3				;Sonderzeichen, Grad Celsius
	st		-x,temp1
	brpl	pos					;N-Flag von der Subtraktion oben noch gltig
	sbr		flags,1<<minus		;Wir haben Minusgrade (das merken wir uns)
	com		temp3				;16Bit negativ -> positiv wandeln (2er Komplement)
	neg		temp2
	sbci	temp3,-1
pos:rcall	div					;Zehntel ins Sram
	ldi		temp1,'.'			;Dezimalpunkt
	st		-x,temp1
	rcall	div					;Einer
	rcall	tstzh				;Zehner
	rcall	tstzh				;Hunderter
	cur		10,0				;Cursor 1. Zeile 10. Zeichen
	rjmp	tx2					;String aufs LCD und Fertig
	;-------------------------------------------------------------------------------------
	;Bei Zehner und Hunderter prfen ob Space, Minuszeichen oder Ziffer in den String muss 
	;-------------------------------------------------------------------------------------
tstzh:
	cp		temp2,null			;Ist noch was da ?
	cpc		temp3,null
	brne	div					;Wenn ja dividieren
	sbrs	flags,minus			;Haben wir ein Minuszeichen im Gepck ?
	rjmp	spc					;Wenn nein, Leerzeichen
	cbr		flags,1<<minus		;Nchste Runde kein Minuszeichen mehr
	ldi		temp1,'-'			;Minuszeichen in den String				
	rjmp	sp1
spc:ldi		temp1,' '			;Leerzeichen in den String		
	rjmp	sp1
	;--------------------------------------------------------------------------------
	;		16Bit Division durch 8Bit Konstante. In diesem Fall 10.
	;		bergaberegister & Ergebniss in temp2/temp3 (low/high), Rest = temp1
	;--------------------------------------------------------------------------------
div:ldi		temp4,16		;16* schieben
	clr		temp1			;Rest =0
dv1:lsl		temp2			
	rol		temp3
	rol		temp1
	cpi		temp1,10		;Solange Bits in temp1 (Rest) schieben bis >= 10
	brcs	dv2				;Wenn der Rest <10 ist, das 0-Bit in temp2 (LSL, oben) stehen lassen
	inc		temp2			;Ansonsten Bit0 setzen und 10 vom Rest abziehen
	subi	temp1,10		
dv2:dec		temp4			;Das ganze 16 mal
	brne	dv1	
	;-------------------------------------------------
	;				Ende Div Routine.
	;Rest der Division wird als ASCii im Sram abgelegt
	;-------------------------------------------------
	subi	temp1,-'0'		;Divisionsrest nach ASCii wandeln
sp1:st		-x,temp1		;und wegspeichern
	ret	
;**********************************************************************************
;						  --- ENDE Thermometer ---
;**********************************************************************************

;LCD - Routine 4-Bit
;AVR	-	LCD
;--------------
;Pd0	-	DB4
;Pd1	-	DB5
;Pd2	-	DB6
;Pd3	-	DB7
;Pd4	-	RS
;Pd5	-	E
;GND	-	R/W	(*nur reinschreiben und die max. Zeit abwarten)
;NC 	-	DB0-DB3
;-------------------------------------------------------------
;LCD-Initialisierung
lcd_init:ldi	temp1,$3f
		out		DDRD,temp1			;PORTD Ausgang
		rcall	wait15				;15ms warten
		ldi		temp1,0b00000011	;Function Set 
		out		PORTD, temp1		;(Bits verschoben da 8 Bit Interface)	
_3mal:	rcall	LCDEnable	
		rcall	wait5				;5ms warten
		dec		temp1				;Fertig ?
		brne	_3mal
		ldi		temp1,0b00000010	;Function Set (noch 8 Bit Modus)
		out		PORTD, temp1		;4 Bit-Interface 
		rcall	LCDEnable
		rcall	wait5				;kurz warten
		ldi		temp1,0b00101000	;Function Set
		rcall	BefOut				;4Bit,2 Zeilen,5*7
		ldi		temp1,0b00001100	
		rcall	BefOut				;
		ldi		temp1,0b00000110	;Entry Mode Set
		rcall	BefOut				;
		ret
;--------------------------------------------------------------------
;Enable Impuls
LCDEnable:
		sbi		PORTD,5				;Enable 1
;		nop							;250nS warten
		nop
		cbi		PORTD,5				;Enable 0
		ret
;-----------------------------------------------------------
;LCD-Befehl / Char ausgeben
BefOut:	clr		temp2				;RS? NEIN
		sbrc	temp2,0				;berspringt immer
AscOut:	ldi		temp2,0b00010000	;Maske RS
		push	temp1				;Brauch ich nochmal
		swap	temp1				;erst mal Bit4-7       
		rcall	Nib1
		pop		temp1
Nib1:	andi	temp1,0b00001111	;den Rest wech	
		or		temp1,temp2			;setzt RS (oder hald net)
;		in		temp3,PORTD
;		cbr		temp3,0b00111111
;		or		temp1,temp3
		out		PORTD, temp1
		rcall	LCDEnable
		rcall	wait50				;braucht man normal nur beim 2. mal 
        ret                            
;------------------------------------------------------------
;String aus SRam ausgeben
tx2:	ld		temp1,x+
		tst		temp1
		breq	extx				; -> ret
		rcall	AscOut
		rjmp	tx2
;------------------------------------------------------------
;String aus Flash ausgeben (Z Pointer)
tx1:	lpm		temp1,z+
		tst		temp1
		breq	extx				; -> ret
		rcall	AscOut
		rjmp	tx1
;------------------------------------------------------------
home:	ldi		temp1,0b00000010	; Cursor Home
		sbrc	temp1,0				;springt immer
;------------------------------------------------------------
clrs:	ldi		temp1,0b00000001	;Display lschen
		rcall	BefOut
		rcall	wait5
extx:	ret
;---------------------------------------------------------
;Warteschleifen fr LCD 8Mhz
wait50:	push	temp1		;50S / Zykl. 2
		ldi		temp1,131	;1
wait:	dec		temp1		;1
		brne	wait		;2
		pop		temp1		;2
		ret					;4
;131*3=393 / +9 = 408 Zyklen * 125 nS (8Mhz) = 51 mS
 
wait15:	rcall	wait5		;15ms
		rcall	wait5
wait5:	push	temp1		;5ms
		ldi		temp1,100
warte:	rcall	wait50
		dec		temp1
		brne	warte
		pop		temp1
		ret
wait1200:rcall	wait600
wait600:rcall	wait300
wait300:push	temp1
		ldi		temp1,20
warte1:	rcall	wait15
		dec		temp1
		brne	warte1
		pop		temp1
		ret
;**************************************************************************************************
;Tabellen
;**************************************************************************************************
;Kalender------------------------------------------------------------------------------------------
wttext:		.db	"MoDiMiDoFrSaSo"						;Wochentage
motext:		.db	"JANFEBMARAPRMAIJUNJULAUGSEPOKTNOVDEZ"	;Monat
molen:		.db 31,28,31,30,31,30,31,31,30,31,30,31		;Tage im Monat
monziffer:	.db	6,2,2,5,0,3,5,1,4,6,2,4					;Monatsziffer zum berechnen des Wochentag
;Uhr und Datum einstellen--------------------------------------------------------------------------
Menutxt:	.db	"** Setup **",0
txt1:		.db	"  Jahr <20  >",0
txt2:		.db	"   Monat <  >",0
txt3:		.db	"     Tag <  >",0
txt5:		.db	" Stunden <  >",0
txt6:		.db	" Minuten <  >",0
txt7:		.db	"<00:00:00> <LOS> ",0
;Wecker -------------------------------------------------------------------------------------------
Alarmtxt:	.db	"  ** Alarm **",0
txt8:		.db	"Alarmton <EIN> ",0
txt9:		.db	"Alarmton <AUS> ",0
;LCD Sonderzeichen 1,2,3---------------------------------------------------------------------------
;Alarm Ein
ch1:
			.db	0b00001,\
				0b00011,\
				0b01101,\
				0b11001,\
				0b11001,\
				0b01101,\
				0b00011,\
				0b00001
;Alarm Aus---------------------------------------------------------------------------------
			.db	0b00000,\
				0b10001,\
				0b01010,\
				0b00100,\
				0b01010,\
				0b10001,\
				0b00000,\
				0b00000
;Grad------------ ---------------------------------------------------------------------------------
			.db	0b00110,\
				0b01001,\
				0b01001,\
				0b00110,\
				0b00000,\
				0b00000,\
				0b00000,\
				0b00000
;***************************************************************************************************
